/*
Class HypDialog

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.*;
import java.awt.event.*;

class HypDialog extends JDialog
                   implements ActionListener    {
    private JTable table;	                   
    private String clause = null;
    private boolean select = false;
	private String[] clauseList;
	private JTextPane selText, inputText;
	private String exp = null;
	
     
    public String getClause() {
        return clause;
    }

    public String getInput() { return exp; }
    	
	public boolean isSelect() { return select; }	
	
		
    public HypDialog(Frame aFrame, String[] cList) {
        super(aFrame, true);

        String[] col = {"Hypotheses"};
        
        
        
        if(cList == null) return;
        
        if(cList.length == 0) return;
        
        clauseList = cList;
        
        
        setTitle("Choose a hypothesis");
		String[][] data = new String[cList.length][1];
		
		for(int i=0; i < cList.length; ++i)
		{
			data[i][0] = cList[i];	
		}
		
		table = new JTable(data, col);
        table.setPreferredScrollableViewportSize(new Dimension(400, 70));

        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        ListSelectionModel rowSM = table.getSelectionModel();
        rowSM.setSelectionInterval(0,0);
        rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
        		//Ignore extra messages.
        		if (e.getValueIsAdjusting()) return;
        
        		ListSelectionModel lsm =
            			(ListSelectionModel)e.getSource();
        		if (lsm.isSelectionEmpty()) {
        			return;    
        		} else {
            		int selectedRow = lsm.getMinSelectionIndex();
            		selText.setText(clauseList[selectedRow]);
            		selText.setCaretPosition(0);
        		}
    		}
		});

        JScrollPane listPane = new JScrollPane(table);
        //listPane.setMinimumSize(new Dimension(400, 100));
        
        JPanel btnPanel = new JPanel();
        
        JButton btnSel = new JButton("Select");
        JButton btnCancel = new JButton("Cancel");
		        
        LincDoc doc = new LincDoc();
        selText = new JTextPane(doc);
        selText.setText(clauseList[0]);
        selText.setCaretPosition(0);
        selText.setMargin(new Insets(5,5,5,5));
		selText.setEditable(false);
		
        exp = "";
        doc = new LincDoc();
        inputText = new JTextPane(doc);
        inputText.setText(exp);
        inputText.setCaretPosition(0);
        inputText.setMargin(new Insets(5,5,5,5));
		
        JLabel lbInput = new JLabel("Enter expression");   
        JLabel lbSel = new JLabel("Selected hypothesis");
        
        JScrollPane curSelView1 = new JScrollPane(selText);
        curSelView1.setMinimumSize(new Dimension(400, 70));
        JPanel curSelView = new JPanel();
        curSelView.setLayout(new BorderLayout());
        curSelView.add(curSelView1, BorderLayout.CENTER);
        curSelView.add(lbSel, BorderLayout.NORTH);
        
        JScrollPane inputView1 = new JScrollPane(inputText);
        inputView1.setMinimumSize(new Dimension(400, 70));
        JPanel inputView = new JPanel();
        inputView.setLayout(new BorderLayout());
        inputView.add(inputView1, BorderLayout.CENTER);
        inputView.add(lbInput, BorderLayout.NORTH);
        
        JSplitPane textView = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        textView.setTopComponent(curSelView);
        textView.setBottomComponent(inputView);
        textView.setDividerLocation(100);
        
        btnSel.setActionCommand("Select");
        btnSel.addActionListener(this);
		btnCancel.setActionCommand("Cancel");
		btnCancel.addActionListener(this);
		
		btnPanel.add(btnSel);
		btnPanel.add(btnCancel);
		
		JPanel textAndBtn = new JPanel();
		
		textAndBtn.setLayout(new BorderLayout());
		textAndBtn.add(textView, BorderLayout.CENTER);
		textAndBtn.add(btnPanel, BorderLayout.SOUTH);
		textAndBtn.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(""), 
			BorderFactory.createEmptyBorder(5,5,5,5)));
		
        JPanel mainPanel = new JPanel();
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(listPane);
        splitPane.setBottomComponent(textAndBtn);
        splitPane.setMinimumSize(new Dimension(500, 300));
        splitPane.setPreferredSize(new Dimension(500,400));
        splitPane.setDividerLocation(150);
        
        
        //Make this dialog display it.
		setContentPane(splitPane);
		table.doLayout();
    }

    
    public void actionPerformed(ActionEvent e) {
	    
        if("Select".equals(e.getActionCommand()))
        {
	         int i = table.getSelectedRow();
	         if(i >= 0)
	         	clause = clauseList[i];
			  else clause = null;	
			  
			  select = true;			   
			  exp = inputText.getText();
			  
			  clearAndHide();    		        
        }
        else if("Cancel".equals(e.getActionCommand()))
        {
	    	clearAndHide();    
        }
    }

    public void clearAndHide() {
        
        setVisible(false);
    }
}

