/*
Class LPrologInterface: interfacing between Java client and lambda prolog 

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

import java.io.*;

public class LPrologInterface
{
	public final static int NO_INPUT = 0;
	public final static int INPUT_MAINFO = 1;
	public final static int INPUT_EXP = 2;
	public final static int INPUT_BOTH = 3;
	
	private boolean isdebug;
	private String def;
	private String seq;
	private String mainFo;
	private String input;
	private String log;
	private boolean result;
		
	public LPrologInterface()
	{
		isdebug = false;
		result = false;
		def = "";
		seq = "";
		mainFo = "";
		input = "";
		log = "";				
		
	}
	
	public void setDefinition(String d) { def = d; }
	public void setSequent(String s) { seq = s; }
	public void setMainForm(String f) { mainFo = f;}
	public void setInput(String i) { input = i; }
	
	public void setDebugMode(boolean mode)
	{
		isdebug = mode;	
	}
	
	public String getLog() { return log; }
	public boolean isQueryOK() { return result; }
	
	
	public boolean callLProlog(String rule)
	{
		String[] ls;
		
		try {
			
			//System.out.println("callLProlog: applyin rule " + rule);
			
			FileWriter rf = new FileWriter(".rule");
			rf.write(rule + "\n");
			rf.close();
			
			// reset synchronization file
			
			FileWriter synf = new FileWriter(".synsvr.txt");
			synf.write("wait");
			synf.close();
			
			Runtime rt = Runtime.getRuntime();
			String[] callAndArgs = {"tjsim", "-b", "--solve", "serve.", "lincserver"};
			
//	       		"tjsim -b --solve \"serve.\" lincserver" ;
//	   		System.out.println("Sending query " + callAndArgs);		       
			Process child = rt.exec(callAndArgs);

			InputStream pin = child.getInputStream();
			log = "";
			int r;
			while((r = pin.read()) != -1)
			{
			   	log = log + (char) r;
		  	}			
		  	
			child.waitFor();   	
			if(child.exitValue() == 0) 
			{
				FileReader inf = new FileReader(".synsvr.txt");
				if(inf.read() == 'o' && inf.read() == 'k')
				{
					log = log + "\nQuery succeeded\n";
					inf.close();
					return true;
				}					
				else {
					log = log + "\nQuery failed\n";
					inf.close();
					return false;
				}
			}
			else 
			{
				log = log + "\nServer terminated abnormally\n";
				return false;
			}
				
		}
		catch (FileNotFoundException exp)
		{
			System.err.println("File .in.seq missing\n");
			return false;
		}
		catch(IOException exp) {
			System.err.println("IOException starting process!");
			return false;
		}
		catch(InterruptedException exp) {
			System.err.println("Interrupted waiting for process!");
			return false;
		}	
		
		
	}
	
	public String[] getHyp(String rule)
	{
		String[] ls = null;
		
		result = false;
		
		try{
			FileWriter outf = new FileWriter(".in.seq");
			outf.write(seq);
			outf.close();
			
			FileWriter synf = new FileWriter(".synsvr.txt");
			synf.write("wait");
			synf.close();
			
			FileWriter def_file = new FileWriter(".cur.def");
			
			// replace carriage-return with space, for compatibility
			// with the lincparser
			String rdef = def.replaceAll("\\r", " ");
			def_file.write(rdef);
			def_file.close();
			
			if(callLProlog("hyp-"+rule)) 
			{
				String s = "";
				int b;
				FileInputStream lseqf = new FileInputStream(".out.seq");
				while((b = lseqf.read()) != -1)
				{
					s = s + (char) b;
				}
				ls = Sequent.parseSequents(s);
				result = true;
			}
		}
		catch (FileNotFoundException exp)
		{
			System.err.println("Missing files\n");
		}
		catch(IOException exp) {
			System.err.println("IOException starting process!");
		}
		return ls;
	}
		
	public String[] execute_inf(String rule, int needInput, String ofname)
    {		
	    String[] ls = null;
	    
	    result = false;
	    
	    try {		    
			FileWriter outf = new FileWriter(".in.seq");
			outf.write(seq);
			outf.close();
			
			FileWriter synf = new FileWriter(".synsvr.txt");
			synf.write("wait");
			synf.close();
			
			FileWriter def_file = new FileWriter(".cur.def");
			String rdef = def.replaceAll("\\r", " ");
			def_file.write(rdef);
			def_file.close();
			

			if(needInput == INPUT_MAINFO || needInput == INPUT_BOTH)
			{
				
				char[] s = mainFo.toCharArray();
				
				for(int i=0; i < s.length; ++i)
				{
					if(s[i] == '\n' || s[i] == '\r') s[i] = ' ';	
				}
				String s1 = "(" + (new String(s)) + ")\n";
				
				FileWriter fhyp = new FileWriter(".main.fo");
				fhyp.write(s1);
				fhyp.close();
			}
			if(needInput == INPUT_EXP || needInput == INPUT_BOTH)
			{
				
				char[] s = input.toCharArray();
				
				for(int i=0; i < s.length; ++i)
				{
					if(s[i] == '\n' || s[i] == '\r') s[i] = ' ';	
				}
				
				String s1 = "(" + (new String(s)) + ")\n";
				
				FileWriter fhyp = new FileWriter(".in.fo");
				fhyp.write(s1);
				fhyp.close();				
			}			
			
			if(callLProlog(rule)) 
			{
				String s = "";
				int b;
				FileInputStream lseqf = new FileInputStream(ofname);
				while((b = lseqf.read()) != -1)
				{
					s = s + (char) b;
				}
				ls = Sequent.parseSequents(s);
				result = true;
			}
		}
		catch (FileNotFoundException exp)
		{
			System.err.println("File .in.seq missing\n");
		}
		catch(IOException exp) {
			System.err.println("IOException starting process!");
		}
		
		return ls;
    }	

}


/*    old source code
	private String[] execute_inf(String rule, int sel, String ofname)
    {		
	    String[] ls = null;
	    Cursor cr = new Cursor(Cursor.WAIT_CURSOR);
	    this.setCursor(cr);
	    
	    try {
			FileWriter outf = new FileWriter("~in.seq");
			seqView.writeFile(outf);								
			outf.close();
			
			FileWriter synf = new FileWriter("~synsvr.txt");
			synf.write("wait");
			synf.close();
			FileWriter def_file = new FileWriter("~cur.def");
			textDef.write(def_file);
			def_file.close();
			

			if(needInput[sel] == 1 || needInput[sel] == 3)
			{
				String s1 = textHyp.getText();
				char[] s = s1.toCharArray();
				
				for(int i=0; i < s.length; ++i)
				{
					if(s[i] == '\n' || s[i] == '\r') s[i] = ' ';	
				}
				s1 = "(" + (new String(s)) + ")\n";
				
				FileWriter fhyp = new FileWriter("~main.fo");
				fhyp.write(s1);
				fhyp.close();
			}
			if(needInput[sel] == 2 || needInput[sel] == 3)
			{
				String s1 = textExp.getText();
				char[] s = s1.toCharArray();
				
				for(int i=0; i < s.length; ++i)
				{
					if(s[i] == '\n' || s[i] == '\r') s[i] = ' ';	
				}
				
				s1 = "(" + (new String(s)) + ")\n";
				
				FileWriter fhyp = new FileWriter("~in.fo");
				fhyp.write(s1);
				fhyp.close();				
			}			
			
			Runtime rt = Runtime.getRuntime();
			String callAndArgs = 
		       "tjsim -b --solve \"serve \\\"" + rule + "\\\".\" lincserver" ;
			System.out.println("Sending query " + callAndArgs);		       
			Process child = rt.exec(callAndArgs);
			// child.waitFor();
			// System.out.println("Process exit code is: " + child.exitValue());
			InputStream pin = child.getInputStream();
			//int r;
			//while((r = pin.read()) != -1)
			//{
			//	char k = (char) r;
			//	Character a = new Character(k);
			//   	System.out.print(a);
		  	//}
		  	textOutput.setText("");
		  	textOutput.read(pin,null);
			child.waitFor();   
			if(child.exitValue() == 0) 
			{
				FileReader inf = new FileReader("~synsvr.txt");
				if(inf.read() == 'o' && inf.read() == 'k')
				{
					String s = "";
					int b;
					FileInputStream lseqf = new FileInputStream(ofname);
					while((b = lseqf.read()) != -1)
					{
						s = s + (char) b;
					}
					ls = Sequent.parseSequents(s);
				}					
				else {
					System.err.println("Query failed");
					textOutput.setText(textOutput.getText() + "Query failed\n");
				}
				inf.close();
			}
			else 
			{
				System.err.println("Server terminated abnormally");				
				textOutput.setText(textOutput.getText() + "Server terminated abnormally\n");
			}
		}
		catch (FileNotFoundException exp)
		{
			System.err.println("File ~in.seq missing\n");
		}
		catch(IOException exp) {
			System.err.println("IOException starting process!");
		}
		catch(InterruptedException exp) {
			System.err.println("Interrupted waiting for process!");
		}	
		
		cr = new Cursor(Cursor.DEFAULT_CURSOR);
		this.setCursor(cr);
		return ls;
    }
*/
