/*
Class Linc (main module)

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// Module: Linc
// Author: Alwen Tiu
// Date: 8/26/03
// Graphical user interface for Linc
// - Provides interface to the Lambda Prolog
// - Executes file lincserver
// - main module.

import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.net.URL;
import java.util.Hashtable;

public class Linc extends JFrame implements ActionListener {
	private JTextPane textDef;
	private LincDoc defDoc;
	private SequentView seqView;
    JTabbedPane tabPane;
    private JTextPane textOutput;
    private JComboBox choice;
	String newline = "\n";
	LPrologInterface lp;
	JLabel drvStatus, defStatus;
	String mainFo, exp;
	Hashtable inpMode;
	
    private static boolean DEBUG = false;
	private DerivationView drvView;
	private boolean nodef;  // is definition loaded or created?
	String[] tactics; 
	Frame topFrame;

	File lastPath;	 	
	File curDefFile, curDrvFile;	
	
    public Linc() {
	    	    
        super("Basic Linc");
        
        mainFo = "";
        exp = "";
        
		nodef = true;
		
		curDefFile = null;
		curDrvFile = null;
		lastPath = null;
		
		initInputMode();
		
        seqView = new SequentView();
        drvView = new DerivationView(this);
        
        tabPane = new JTabbedPane();
        tabPane.add("Query", seqView);
        textOutput = new JTextPane();
        textOutput.setEditable(false);
        JScrollPane outputView = new JScrollPane(textOutput);
        tabPane.add("Console", outputView);

        defDoc = new LincDoc();
        textDef = new JTextPane(defDoc);
        textDef.setEditable(true);
              	
      
        //Add the scroll panes to a split pane.
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setLeftComponent(drvView);
        splitPane.setRightComponent(tabPane);

        Dimension minimumSize = new Dimension(500, 100);
        seqView.setMinimumSize(minimumSize);
        tabPane.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(150);
		
        splitPane.setPreferredSize(new Dimension(800, 480));
		splitPane.setOneTouchExpandable(true);        
		
                		        
        drvStatus = new JLabel("Derivation file: [new]");
        defStatus = new JLabel("Definition file: [none]");
        
        JPanel statusPane = new JPanel(new GridLayout(1, 0));
        statusPane.add(drvStatus);
        statusPane.add(defStatus);

        JToolBar toolBar1 = new JToolBar();        
        JToolBar toolBar2 = new JToolBar();
        
        makeCoreToolbar(toolBar1);
        makeExtToolbar(toolBar2);
        
        JPanel tbView = new JPanel();
        tbView.setLayout(new GridLayout(0,1));
        tbView.add(toolBar1);
        tbView.add(toolBar2);
        
        getContentPane().setLayout(new BorderLayout());
        
        getContentPane().add(splitPane, BorderLayout.CENTER);
        getContentPane().add(tbView, BorderLayout.NORTH);
        getContentPane().add(statusPane, BorderLayout.SOUTH);
        
    	lp = new LPrologInterface(); 	
    	
        setJMenuBar(createMenuBar());   
        drvView.newDerivation();

        URL url = Linc.class.getResource("tao.gif"); 
        Image img = Toolkit.getDefaultToolkit().getImage(url);
        setIconImage(img);
        
        topFrame = (Frame)this;
    }

    
    protected void initInputMode()
    {
		inpMode = new Hashtable();
		String[] ruleNm = 
		   {"and-l", "and-r", 
		   "coind-r", "contr-l", 
		   "def-l", "def-r", 
		   "def-r-clause", "eq-l", 
		   "eq-r", "bot", 
		   "focus", "focus-eql", "imp-l", 
		   "imp-r", "ind-l", 
		   "init", "or-l", 
		   "or-r1", "or-r2", 
		   "pi-l", "pi-r",
		   "sigma-l", "sigma-r", 
		   "top", "weak-l"};
		   
		int[] inp = 
		      {LPrologInterface.INPUT_MAINFO, LPrologInterface.NO_INPUT, 
		       LPrologInterface.INPUT_EXP, LPrologInterface.INPUT_MAINFO, 
		       LPrologInterface.INPUT_MAINFO, LPrologInterface.NO_INPUT, 
		       LPrologInterface.NO_INPUT,  LPrologInterface.INPUT_MAINFO,
		       LPrologInterface.NO_INPUT, LPrologInterface.NO_INPUT, 
		       LPrologInterface.NO_INPUT, LPrologInterface.NO_INPUT, LPrologInterface.INPUT_MAINFO, 
		       LPrologInterface.NO_INPUT, LPrologInterface.INPUT_BOTH, 
		       LPrologInterface.NO_INPUT, LPrologInterface.INPUT_MAINFO, 
		       LPrologInterface.NO_INPUT, LPrologInterface.NO_INPUT, 
		       LPrologInterface.INPUT_BOTH, LPrologInterface.NO_INPUT, 
		       LPrologInterface.INPUT_MAINFO, LPrologInterface.INPUT_EXP, 
		       LPrologInterface.NO_INPUT, LPrologInterface.INPUT_MAINFO};
		     
		int n = ruleNm.length;
		
		for(int i=0; i < n; ++i)
		{
			inpMode.put(ruleNm[i], new Integer(inp[i]));	
		}  		
		inpMode.put("def-r-inst", new Integer(LPrologInterface.NO_INPUT));
		inpMode.put("def-r-one", new Integer(LPrologInterface.NO_INPUT));
    }
    
    protected ImageIcon getImgRes(String res)
    {
		URL url = Linc.class.getResource(res); 			
		Image img = Toolkit.getDefaultToolkit().getImage(url); 
		ImageIcon icon = new ImageIcon(img);
		return icon;
    }
    
    protected void makeExtToolbar(JToolBar toolBar)
    {
        JButton button = null;

        toolBar.setFloatable(false);
        
        //button = new JButton(new ImageIcon("images/defleft.jpg"));
        button = new JButton(getImgRes("images/defleft.jpg"));
        button.setToolTipText("Definition left");
        button.setActionCommand("rule def-l");
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/defright.jpg"));
        button = new JButton(getImgRes("images/defright.jpg"));
        button.setToolTipText("Definition right");
        button.setActionCommand("rule def-r");
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/eqleft.jpg"));
        button = new JButton(getImgRes("images/eqleft.jpg"));
        button.setToolTipText("Equality left (unification)");
        button.setActionCommand("rule eq-l");        
        button.addActionListener(this);
        toolBar.add(button);

        //button = new JButton(new ImageIcon("images/eqright.jpg"));
        button = new JButton(getImgRes("images/eqright.jpg"));
        button.setToolTipText("Equality right");
        button.setActionCommand("rule eq-r");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/indleft.jpg"));
        button = new JButton(getImgRes("images/indleft.jpg"));
        button.setToolTipText("Induction");
        button.setActionCommand("rule ind-l");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/coindright.jpg"));
        button = new JButton(getImgRes("images/coindright.jpg"));
        button.setToolTipText("Co-induction");
        button.setActionCommand("rule coind-r");
        button.addActionListener(this);
        toolBar.add(button);

        JPanel selectPanel = makeSelectPanel();
        	    
        toolBar.add(selectPanel);
    }
    
    protected void makeCoreToolbar(JToolBar toolBar) {
        JButton button = null;

        toolBar.setFloatable(false);
        
        //button = new JButton(new ImageIcon("images/topright.jpg"));
        button = new JButton(getImgRes("images/topright.jpg"));
        button.setToolTipText("True");
        button.setActionCommand("rule top");        
        button.addActionListener(this);
        toolBar.add(button);

        //button = new JButton(new ImageIcon("images/botleft.jpg"));
        button = new JButton(getImgRes("images/botleft.jpg"));
        button.setToolTipText("False");
        button.setActionCommand("rule bot");        
        button.addActionListener(this);
        toolBar.add(button);
        
        
        //button = new JButton(new ImageIcon("images/andleft.jpg"));
        button = new JButton(getImgRes("images/andleft.jpg"));
        button.setToolTipText("And-left");
        button.setActionCommand("rule and-l");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/andright.jpg"));
        button = new JButton(getImgRes("images/andright.jpg"));
        button.setToolTipText("And-right");
        button.setActionCommand("rule and-r");        
        button.addActionListener(this);
        toolBar.add(button);


        //button = new JButton(new ImageIcon("images/orleft.jpg"));
        button = new JButton(getImgRes("images/orleft.jpg"));
        button.setToolTipText("Or-left");
        button.setActionCommand("rule or-l");
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/orright1.jpg"));
        button = new JButton(getImgRes("images/orright1.jpg"));
        button.setToolTipText("Or-right1");
        button.setActionCommand("rule or-r1");        
        button.addActionListener(this);
        toolBar.add(button);

        //button = new JButton(new ImageIcon("images/orright2.jpg"));
        button = new JButton(getImgRes("images/orright2.jpg"));
        button.setToolTipText("Or-right2");
        button.setActionCommand("rule or-r2");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/impleft.jpg"));
        button = new JButton(getImgRes("images/impleft.jpg"));
        button.setToolTipText("Implication-left");
        button.setActionCommand("rule imp-l");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/impright.jpg"));
        button = new JButton(getImgRes("images/impright.jpg"));
        button.setToolTipText("Implication-right");
		button.setActionCommand("rule imp-r");                
        button.addActionListener(this);
        toolBar.add(button);

        //button = new JButton(new ImageIcon("images/existsleft.jpg"));
        button = new JButton(getImgRes("images/existsleft.jpg"));
        button.setToolTipText("Existential-left");
		button.setActionCommand("rule sigma-l");                
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/existsright.jpg"));
        button = new JButton(getImgRes("images/existsright.jpg"));
        button.setToolTipText("Existential-right");
        button.setActionCommand("rule sigma-r");        
        button.addActionListener(this);
        toolBar.add(button);
        
        
        //button = new JButton(new ImageIcon("images/allleft.jpg"));
        button = new JButton(getImgRes("images/allleft.jpg"));
        button.setToolTipText("Universal-left");
        button.setActionCommand("rule pi-l");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/allright.jpg"));
        button = new JButton(getImgRes("images/allright.jpg"));
        button.setToolTipText("Universal-right");
        button.setActionCommand("rule pi-r");        
        button.addActionListener(this);
        toolBar.add(button);


        //button = new JButton(new ImageIcon("images/id.jpg"));
        button = new JButton(getImgRes("images/id.jpg"));
        button.setToolTipText("Initial");
        button.setActionCommand("rule init");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/weakleft.jpg"));
        button = new JButton(getImgRes("images/weakleft.jpg"));
        button.setToolTipText("Weakening");
        button.setActionCommand("rule weak-l");        
        button.addActionListener(this);
        toolBar.add(button);
        
        //button = new JButton(new ImageIcon("images/contrleft.jpg"));
        button = new JButton(getImgRes("images/contrleft.jpg"));
        button.setToolTipText("Contraction");
        button.setActionCommand("rule contr-l");        
        button.addActionListener(this);
        toolBar.add(button);

    }
        
    public void displaySequent(Sequent s)
    {
		seqView.displaySequent(s);
		tabPane.setSelectedIndex(0);
    }
    
    public String getSequent()
    {
	 	return seqView.getSequent();   
    }
    
    private JPanel makeSelectPanel()
    {
        JPanel selectPanel = new JPanel();
		choice = null;
		
		String[] rules = 
		   {
			   "definition-right with instantiation",
			   "focusing",
			   "focusing with equality left",
	    	};
		
		String[] ruleNm = 
		   {"def-r-clause", "focus", "focus-eql"};
		   		     		     
		tactics = ruleNm;

			
		JLabel lbSel = new JLabel("Tactics ");
		
		JButton applyBtn = new JButton("Apply");        	
		applyBtn.setActionCommand("Linc-apply");
		applyBtn.addActionListener(this);
		
		choice = new JComboBox(rules);
		choice.setSelectedIndex(0);
		choice.addActionListener(this);
		
		selectPanel.setLayout(new FlowLayout(FlowLayout.LEFT));		
		selectPanel.add(lbSel);
		selectPanel.add(choice);
		selectPanel.add(applyBtn);
					
   		return selectPanel;
    }

    private Frame getCurFrame()
    {
	 	return (Frame) (this);   
    }
    


	private void newDefinition()
	{
	     	if(nodef)
	     	{
        		JScrollPane defView = new JScrollPane(textDef);
        		tabPane.add("Definition", defView);					
        		nodef = false;
	     	}
	     	textDef.setText("");
	     	
	     	tabPane.setSelectedIndex(2);		
	     	curDefFile = null;
	}
	
	private void loadDefinition(File fs)
	{
		try {
			FileReader inf = new FileReader(fs);
						
			String s = "";			
			
			int b;			
			int offset = 0;
			char[] cbuf = new char[10240];
						
			MutableAttributeSet attr = new SimpleAttributeSet();
	       	StyleConstants.setFontFamily(attr, "Courier");
    	   	StyleConstants.setFontSize(attr, 14); 
    	   	
    	   	try{
	    	   	b = inf.read(cbuf, 0, 10240);
	    	   	if(b != -1) textDef.setText("");
	    	   	
	    	   	Cursor cur = new Cursor(Cursor.WAIT_CURSOR);
	    	   	this.setCursor(cur);
				while(b != -1)
				{
					s = new String(cbuf, 0, b);
					defDoc.insertString(offset, s, attr);
					offset = offset + b;
					b = inf.read(cbuf, 0, 10240);
				}
				cur = new Cursor(Cursor.DEFAULT_CURSOR);
				this.setCursor(cur);
			}
			catch(BadLocationException e)
			{
				this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				errMsg("Error loading file");	
			}
			
			if(nodef)
			{ // if this is the first time the definition is loaded
        		JScrollPane defView = new JScrollPane(textDef);
        		tabPane.add("Definition", defView);					
        		nodef = false;
			}				
			inf.close();
			tabPane.setSelectedIndex(2);
			curDefFile = fs;
		}
		catch(IOException e)
		{
			String m;
			m = "Error in loading file " + fs.getPath();
			errMsg(m);
		}
	}

	private void saveDefinition(File fs)
	{
		try {
			if(nodef) return;
			FileWriter outf = new FileWriter(fs);
			//textDef.write(outf);
			String s = textDef.getText();
			s = s.replaceAll("\\r", " ");
			
			outf.write(s);
			outf.close();
			curDefFile = fs;
		}
		catch(IOException e)
		{
			String m;
			m = "Error in writing file " + fs.getPath();
			errMsg(m);
		}	
	}	
	
	private void newTreeModel()
	{
		
		drvView.newDerivation();

		curDrvFile = null;
		
	}
	
	private void saveTreeModel(File fpath)
	{
        if(drvView.writeFile(fpath))
           	curDrvFile = fpath;		
	}
	
	private void loadTreeModel(File fpath)
	{		
		if(drvView.loadFile(fpath) == false) return;
		
		seqView.setSequent("");
		curDrvFile = fpath;
	}

    private void execute_defR(String[] ls)
    {									
	    			    	
		DefDialog dd;
		
		dd = new DefDialog(this, ls);
		dd.pack();
		dd.show();


		
		if(dd.isSelectAll() == false && dd.getClause() != null)
		{
			
			String exp = dd.getClause();
			exp = "(" + exp + ")\n";
						
			char[] sarr = exp.toCharArray();
				
			for(int i=0; i < sarr.length; ++i)
				{
					if(sarr[i] == '\n' || sarr[i] == '\r') sarr[i] = ' ';	
				}
			exp = "(" + (new String(sarr)) + ")\n";
			//System.out.println("Clause: " + exp);

			try{
				FileWriter ff = new FileWriter(".main.fo");
				ff.write(exp);
				ff.close();
			}
			catch(IOException e)
			{ 	
				errMsg("Error in writing file .main.fo");
				return;
			}
			
			String callStr;
			if(dd.isSelect()) callStr = "def-r-select";
			else callStr = "def-r-one";
			
			
			if(lp.callLProlog(callStr))
			{
				//System.out.println("Applying " + callStr);
				try {
					FileReader ff = new FileReader(".out.seq");
					String s = "";
					String[] seqs;
					int b;
				
					while((b = ff.read()) != -1)
					{
						s = s + (char) b;
					}
					seqs = Sequent.parseSequents(s);					
					ff.close();
					drvView.addNewSequents(callStr,"","",seqs);
					textOutput.setText(lp.getLog());
				}
				catch(IOException e)
				{
					
					errMsg("Error in reading file .out.seq");
						
				}
			}									
			
		}	
		else if(dd.isSelectAll())
		{	
			String[] seqs =	execute_inf("def-r-inst", ".out.seq");
			if(seqs != null) drvView.addNewSequents("def-r-inst", "","",seqs);
			
		}				
		else {
			errMsg("Query failed! See console for details.");
			textOutput.setText(textOutput.getText() + "Query failed\n");
		}
    }

    private String[] execute_inf(String rule, String ofname)
    {
		String[] ls = null;
		Integer mode = (Integer) inpMode.get(rule);

		if(mode == null) return null;
		
		int val = mode.intValue();
		mainFo = "";
		//exp = "";
		
		lp.setDefinition(textDef.getText());
		lp.setSequent(seqView.getSequent());
		
		if(val == LPrologInterface.INPUT_MAINFO ||
		   val == LPrologInterface.INPUT_BOTH)
		{
			ls = lp.getHyp(rule);
			textOutput.setText(lp.getLog());
			if(lp.isQueryOK() == false)
			{
				errMsg("Could not find hypotheses.");
				return null;	
			}
			HypDialog dd = new HypDialog(this, ls);			
			dd.pack();
			dd.show();
			if(dd.isSelect())
			{
				lp.setMainForm(dd.getClause());
				mainFo = dd.getClause();
				exp = dd.getInput();
				//textHyp.setText(dd.getClause());
				lp.setInput(dd.getInput());				
			}
			else
			{
				return null;	
			}			
			ls = null;
		}
		else if(val == LPrologInterface.INPUT_EXP)
		{
			InputExpDialog dd = new InputExpDialog(this);
			dd.setInput(exp);
			dd.pack();
			dd.show();
			if(dd.isOK())
			{
				lp.setInput(dd.getInput());
				exp = dd.getInput();
			}	
			else
			{
				return null;	
			}						
		}
		else exp = "";
		
		Cursor cr = new Cursor(Cursor.WAIT_CURSOR);
		this.setCursor(cr);

		ls = lp.execute_inf(rule, val, ofname);    
		cr = new Cursor(Cursor.DEFAULT_CURSOR);
		this.setCursor(cr);
		textOutput.setText(lp.getLog());
		if(lp.isQueryOK() == false)
			errMsg("Query failed! See console for details.");
			
			
		return ls;
    }
    
    
// --------------- menu ---------------------------
    public JMenuBar createMenuBar() {
        JMenuBar menuBar;
        JMenu menu, submenu;
        JMenuItem menuItem;
        JRadioButtonMenuItem rbMenuItem;
        JCheckBoxMenuItem cbMenuItem;

        //Create the menu bar.
        menuBar = new JMenuBar();

        //Build the first menu.
        menu = new JMenu("Derivation");
        menu.setMnemonic(KeyEvent.VK_V);
        menuBar.add(menu);

        //a group of JMenuItems
        menuItem = new JMenuItem("New derivation",
                                 KeyEvent.VK_N);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem("Load derivation");
        menuItem.setMnemonic(KeyEvent.VK_L);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem("Save derivation");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem("Save As derivation");
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        
        menu.addSeparator();

        menuItem = new JMenuItem("Exit");
        menuItem.setMnemonic(KeyEvent.VK_E);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        
        // Definition menu
        menu = new JMenu("Definition");
        menu.setMnemonic(KeyEvent.VK_D);
        menuBar.add(menu);

		menuItem = new JMenuItem("New Definition");
        menuItem.setMnemonic(KeyEvent.VK_N);
        menuItem.addActionListener(this);
        menu.add(menuItem);

		menuItem = new JMenuItem("Load Definition");
        menuItem.setMnemonic(KeyEvent.VK_L);
        menuItem.addActionListener(this);
        menu.add(menuItem);

		menuItem = new JMenuItem("Save Definition");
        menuItem.setMnemonic(KeyEvent.VK_S);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem("Save As Definition");
        menuItem.setMnemonic(KeyEvent.VK_A);
        menuItem.addActionListener(this);
        menu.add(menuItem);                        
        
        //Build help menu in the menu bar.
        menu = new JMenu("Help");
        menu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(menu);
        
        menuItem = new JMenuItem("Content");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        
        return menuBar;
    }

    protected File chooseFile(int type, String title)
    {
	    File f = null;
	    
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogType(type);
        chooser.setDialogTitle(title);
        
        if(lastPath != null)
        {
	        chooser.setCurrentDirectory(lastPath);
        }
        else
        {
	     	File curDir = new File(".");
	     	chooser.setCurrentDirectory(curDir);   
        }
        
    	int returnVal = chooser.showOpenDialog(this);
    	if(returnVal == JFileChooser.APPROVE_OPTION) {       		
            f = chooser.getSelectedFile();      
            lastPath = f;      
    	}
    	return f;
	}
	
    public void actionPerformed(ActionEvent e) {
        
	    String cmd = e.getActionCommand();

	    boolean ruleApp = false;
	    String rule = "";
	    	    
        if (cmd.indexOf("Linc-apply") == 0)
        {
	    	ruleApp = true;
	    	int n = choice.getSelectedIndex();
	    	rule = tactics[n];    
        }
        else if(cmd.indexOf("rule ") == 0)
        {
	     	ruleApp = true;
	     	rule = cmd.substring(5);   
        }
        
        if(ruleApp)
        {
	        if(nodef)
		    	{
			    	errMsg("No Definition loaded yet!");			    	
			    	return;
		    	}
		    
			String[] ls;
		    
			DefaultMutableTreeNode cur_node = drvView.getCurrentNode();
			
			if(cur_node == null)
			{
				errMsg("No sequent selected!");
				return;
					
			}	
			
			Sequent seq = (Sequent) cur_node.getUserObject();
            seq.parse(seqView.getSequent());
            cur_node.setUserObject(seq);

            
		    //System.out.println("\nApplying rule: " + rule);
   		    if("def-r-clause".equals(rule))
		    {
		    	ls = execute_inf(rule, ".in.fo");	
			    if(ls == null) 
			    {
				    errMsg("def-r-clause ls null");
				    return;
			    }
			    
			    execute_defR(ls);
			    return;
		    }
			
			ls = execute_inf(rule, ".out.seq");
		    if(ls == null) return;
		    
			drvView.addNewSequents(rule, mainFo, exp, ls);	    	
		    return;
		}        
		
		if ("comboBoxChanged".equals(e.getActionCommand()))
		{			
			return;
		}
		
		
		JMenuItem source = (JMenuItem)(e.getSource());
        if("New derivation".equals(source.getText()))
        {
	        newTreeModel();
	        drvStatus.setText("Derivation file: [new]");
	        
        }
        else if ("Save derivation".equals(source.getText()))
        {
	     	if(curDrvFile == null)
	     	{
		    	//FileDialog fd = new FileDialog(getCurFrame(), 
		        //   "Save derivation file", FileDialog.SAVE);
		    	//fd.setLocationRelativeTo(this);
		    	//fd.show();
		    
			    //String fname = fd.getFile();
			    //if(fname == null) return;
			    //File fpath = new File(fd.getDirectory(), fname);
			    
			    File fpath = chooseFile(JFileChooser.SAVE_DIALOG,
			                         "Save derivation file");
			    if(fpath != null) {
				    saveTreeModel(fpath);		        		     	
				    drvStatus.setText("Derivation file: " + fpath.getName());
				}
	     	}
	     	else saveTreeModel(curDrvFile);   
        }
        else if ("Save As derivation".equals(source.getText()))
        {
		    //FileDialog fd = new FileDialog(this, "Save derivation file", FileDialog.SAVE);
		    //fd.setLocationRelativeTo(this);
		    //fd.show();

		    //String fname = fd.getFile();
		    //if(fname == null) return;
		    
		    //System.out.println(fd.getFile());
		    //File fpath = new File(fd.getDirectory(), fname);
		    
		    File fpath = chooseFile(JFileChooser.SAVE_DIALOG, "Save derivation file");
		    if(fpath != null) {
			    saveTreeModel(fpath);		        
			    drvStatus.setText("Derivation file: " + fpath.getName());
			}
        }
        else if ("Load derivation".equals(source.getText()))
        {
	        //FileDialog fd = new FileDialog(topFrame, "Load derivation file", FileDialog.LOAD);	        	        
		    
		    //fd.show();
		    //String fname = fd.getFile();
		    		    
		    //if(fname == null) return;
		    //File fpath = new File(fd.getDirectory(), fname);
		    
		    File fpath = chooseFile(JFileChooser.OPEN_DIALOG, "Load derivation file");
		    if(fpath != null) {
			    loadTreeModel(fpath);	
			    drvStatus.setText("Derivation file: " + fpath.getName());
		    }
        }
        else if ("New Definition".equals(source.getText()))
        {
	        newDefinition();
	        defStatus.setText("Definition file: [new]");
        }
        else if ("Load Definition".equals(source.getText()))
        {
	        //FileDialog fd = new FileDialog(getCurFrame(), "Load definition file", FileDialog.LOAD);
		    //fd.show();
		    //String fname = fd.getFile();
		    //if(fname == null) return;
		    //File fpath = new File(fd.getDirectory(), fname);
		    
		    File fpath = chooseFile(JFileChooser.OPEN_DIALOG, "Load definition file");
		    if(fpath != null) {
			    loadDefinition(fpath);		        
			    defStatus.setText("Definition file: " + fpath.getName());
		    }
        }
        else if ("Save Definition".equals(source.getText()))
        {
	     	if(curDefFile == null)
	     	{
		        //FileDialog fd = new FileDialog(getCurFrame(), 
		        //	"Save definition file", FileDialog.SAVE);
			    //fd.show();
			    //String fname = fd.getFile();
			    //if(fname == null) return;

		    	//File fpath = new File(fd.getDirectory(), fname);
		    	
		    	File fpath = chooseFile(JFileChooser.SAVE_DIALOG, "Save definition file");
		    	if(fpath != null) {
			    	saveDefinition(fpath);		     	
			    	defStatus.setText("Definition file: " + fpath.getName());
		    	}
	     	}
	     	else saveDefinition(curDefFile);
	     
        }
        else if ("Save As Definition".equals(source.getText()))
        {
	        //FileDialog fd = new FileDialog(getCurFrame(), "Save definition file", FileDialog.SAVE);
		    //fd.show();
		    //String fname = fd.getFile();
		    //if(fname == null) return;

		    //File fpath = new File(fd.getDirectory(), fname);
		    
		    File fpath = chooseFile(JFileChooser.SAVE_DIALOG, "Save definition file");
		    if(fpath != null) {
			    saveDefinition(fpath);
			    defStatus.setText("Definition file: " + fpath.getName());
		    }
        }
        else if("Content".equals(source.getText()))
        {
	        HelpDialog d = new HelpDialog(this);
	        d.pack();
	        d.show();
        }
        else if ("Exit".equals(source.getText()))
        {
	     	System.exit(0);   
        }
      
    }


    
    protected class HelpDialog extends JDialog
    {
	 	public HelpDialog(Frame fr)
	 	{
        
		 	super(fr, false);
		 	JEditorPane txt;
		 	String s = null;
		 	try{
		 		txt = new JEditorPane();
		 		txt.setEditable(false);
				
        
		 		URL url = Linc.class.getResource("help/introduction.htm"); 			
		 		
            	//s = "file:"
                //	+ System.getProperty("user.dir")
                //	+ System.getProperty("file.separator")
                //	+ "help/introduction.htm";
            	//URL helpURL = new URL(s);
            	//txt.setPage(helpURL);
            	txt.setPage(url);
        	} catch (Exception e) {
            	errMsg("Couldn't open help URL: " + s);
            	return;
        	} 	
					 		
		 	JScrollPane txtView = new JScrollPane(txt);
		 	txtView.setPreferredSize(new Dimension(600, 400));
		 	
		 	getContentPane().setLayout(new BorderLayout());
		 	getContentPane().add(txtView, BorderLayout.CENTER);
	 	}   
	 	
    }
    
    
// ----------------end of menu ----------------------    
	protected void errMsg(String m)
	{
		JOptionPane.showMessageDialog(null,  m, "Error", JOptionPane.ERROR_MESSAGE); 
	}

	public static void main(String[] args) {
		JFrame.setDefaultLookAndFeelDecorated(true);
        JFrame frame = new Linc();
 
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });  

        frame.pack();        
        frame.setVisible(true);
    }
}

