/*
Class LincDoc

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


import javax.swing.*;
import javax.swing.text.*;
import java.util.Vector;
import java.awt.Color;

public class LincDoc extends DefaultStyledDocument
    {
	    MutableAttributeSet defAttr;
	    	 
	    boolean isLincSymbol(char c)
	    {
		 	if(Character.isLetterOrDigit(c))   return true;
		 	if(c == '%' || c == '&' || c == ';' || c == '=' || 
		 	   c == '>' || c == '<') return true;
		 	return false;
	    } 
	    
	    // syntax highlighting
	    
	    public void highlight(int offset, int strLen)
	    {
			int l = getLength();
			
			if(l == 0) return;
			
			String s;
			try{
		 		s = getText(0, l);
	 		}
	 		catch(BadLocationException e)
	 		{
		 		System.out.println("BadLocation exception");
		 		return;	
	 		}
		 			 	
		 	int i;
		 	
		 	if(offset > 0) i = offset - 1;
		 	else i = 0;
		 	
		 	while(i >= 0)
		 	{
			 	//if(!isLincSymbol(s.charAt(i))) break;
			 	if(s.charAt(i) == '\n') break;			 	
			 	--i;
			}
			
			if(i < 0) i = 0;
			
			int j = offset + strLen + 1;
			if(j >= l) j = l-1;
			if(j < 0) j = 0;

			while(j < l)
			{
				//if(!isLincSymbol(s.charAt(j))) break;
				if(s.charAt(j) == '\n') break;
				++j;	
			}
			
			if(j == l) --j;
			if(j < 0) j = 0;
			
			String scanstr = s.substring(i, j);
			
			if(scanstr.length() == 0) 
				return;
				
			MutableAttributeSet attr;
			
			attr = new SimpleAttributeSet();
	       	StyleConstants.setFontFamily(attr, "Courier");
    	   	StyleConstants.setFontSize(attr, 14); 
    	   	StyleConstants.setForeground(attr,Color.black);
    	   	StyleConstants.setBold(attr, false);    
    	   	StyleConstants.setItalic(attr, false);
			
    	   	setCharacterAttributes(i, (j - i), attr, true);
    	   	
    	   	    	      	   	    	   	
    	   	Scanner scanner = new Scanner();
    	   	
    	   	scanner.scanString(scanstr);
    	   	
    	   	Vector pos = scanner.pos;
    	   	Vector len = scanner.len;
    	   	Vector cat = scanner.cat;
    	   	
    	   	for(int k=0; k < pos.size(); ++k)
    	   	{
	    	 	Integer p = (Integer) pos.elementAt(k);
	    	 	Integer li = (Integer) len.elementAt(k);
	    	 	Integer ci = (Integer) cat.elementAt(k);
	    	 	
				attr = new SimpleAttributeSet();
	       		StyleConstants.setFontFamily(attr, "Courier");
    	   		StyleConstants.setFontSize(attr, 14); 
    	   		StyleConstants.setBold(attr, true);
    	   		StyleConstants.setItalic(attr, false);    
    	   		if(ci.intValue() == 0)
    	   			StyleConstants.setForeground(attr, Color.red.darker());
    	   		else if(ci.intValue() == 1)
    	   			StyleConstants.setForeground(attr, Color.blue);
    	   		else if(ci.intValue() == 2){
	    	   		StyleConstants.setForeground(attr, Color.darkGray);
	    	   		StyleConstants.setItalic(attr, true);
    	   		}
    	   		else StyleConstants.setForeground(attr, Color.blue);
	    	   		
    	   		
	    	 	if(ci.intValue() == 2)
	    	 	{
		    	 	int n = 0;
		    	 	
		    	 	for(int ii = p.intValue() + i; ii < l; ++ii)
		    	 	{
		    	 		if(s.charAt(ii) != '\n') ++n;
		    	 		else break;
	    	 		}
		    	 		
	    	 		setCharacterAttributes(p.intValue() + i, n, attr, true);
    	 		}
	    	 	else
	    	 		setCharacterAttributes(p.intValue() + i, li.intValue(), attr, true);
	    	 	  	
    	   	}		    
	    }
	    
	          
	    public void insertString(int offset,
                    String str,
                    AttributeSet a)
        throws BadLocationException
		{
		 	defAttr = new SimpleAttributeSet();
		 	
	       	StyleConstants.setFontFamily(defAttr, "Courier");
    	   	StyleConstants.setFontSize(defAttr, 14);     
			super.insertString(offset, str, defAttr);
			
			highlight(offset, str.length());			
		}    
		
		protected void postRemoveUpdate(AbstractDocument.DefaultDocumentEvent chng)		
		{
			super.postRemoveUpdate(chng);
			int offset = chng.getOffset();
			highlight(offset, 0);					
		}                            
    }    