% Module lincgrammar
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% Module: lincgrammar
% Author: Alwen Tiu
% Date: 8/26/03
% Description: grammar specification of lambda terms, sequents and definitions.
%    Making use of lambdayacc module.

sig lincgrammar.
accum_sig lyaccshares, blists, lincsig.

% terminal symbols for lambda terms
type lparen, rparen, linc_symAbs, linc_symTrue,
     linc_symFalse, linc_symAnd, linc_symOr, linc_symImp, 
     linc_symAll, linc_symExists, linc_colon, linc_comma, linc_dot,
     linc_symEq, linc_symFormt, telparen, linc_symInd, linc_symCoInd gs.

          
% non-terminals
type linc_term tm -> gs.
type linc_makeAbs string -> tm -> (tm -> tm) -> o.


% types

% terminals
type linc_symArr gs.

%nonterminals
type linc_type ty -> gs.


% kind and type declarations

type linc_symKind, linc_symType gs.

type id_list list string -> gs.
type nontermst list string -> gs.

type linc_typeDcl, linc_typeDcls list signat -> gs.
type linc_kindDcl, linc_kindDcls list ty -> gs.

type linc_makeTypeList list string -> ty -> list signat -> o.
type linc_makeKindList list string -> list ty -> o.

% Definitions

% terminal symbols for definitions
% syntax of definitions:
%  def <name>.
%  kind <type> type.
%  type <term> <type>.
%  coind/ind head <=> body

kind linc_def type.
kind linc_defRoot type.

type linc_symDef, linc_symDefName gs.

type linc_definition string -> list linc_def -> linc_defRoot.

type linc_defClause defclause -> gs.
type linc_defHead tm -> gs.
type linc_defBody tm -> gs.
type linc_defn linc_defRoot -> gs.
type linc_defnb list linc_def -> gs.

type linc_kindDec list ty -> linc_def.
type linc_typeDec list signat -> linc_def.
type linc_defDec  defclause -> linc_def. 

% sequents
% syntax:
% seq <sequent's name>
% type <fvar> <type expression>.
% .
% .
% hyp <list of formulas>.
% goal formula.

kind linc_seqt type.

type linc_seqRoot string -> list linc_seqt -> tm -> linc_seqt.

type linc_seqHyp  list tm -> linc_seqt.
type linc_seqSig  list signat -> linc_seqt.

type linc_symSeq, linc_symHyp, linc_symGoal gs.

type linc_sequent linc_seqt -> gs.
type linc_seqList list linc_seqt -> gs.
type linc_seqBody list linc_seqt -> gs.
type linc_seqBody2 tm -> gs.
type linc_tmList  list tm -> gs.


% start symbol

type linc_root A -> gs.

type lincparser o.
