% Module lincserver
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: lincserver
% Author: Alwen Tiu
% Date: 8/26/03
% Description: 
%  - interface module to Java GUI.
%  - main predicate: serve Rule.
%  - synchronize with Java client through several files:
%       .in.seq : input sequent
%       .out.seq : output sequent
%       .cur.def : definition file
%       .synsvr.txt: file to indicate result of query, whether it is a success or a failure.
%               The Java client will overwrite this file with a "wait" string before
%               calling this module. Upon a successful execution, an "ok" string is written
%               to this file by current module.
%       .main.fo : main formula of the sequent
%       .in.fo   : file containing expression, e.g., from user input
%       .rule : contains the rule to apply to the sequent
%  - this module basically invokes predicates in infer.mod for performing
%    various inference rules.  

% Revision:
% - 8/31/03: * add option for extended focusing search (i.e., with eq-l).

module lincserver.

accumulate list, stt, control, infer, printer.

constant X :- 
   current_def (def Nm Kn Sig Cls), 
   member X Sig.
   
%serve Rule :-    
%   load_def ".cur.def" Def,
%   (current_def Def => 
%      (
%          load_seq ".in.seq" [Seq|L] _, !,
%          output std_out "Input sequent:\n",
%          print_ListSequent std_out [Seq],     
%          output std_out "Applying rule: ", print Rule, print "\n",      
%          infer Rule Seq LSeq,
%          output std_out "ok\n",
%          append L LSeq L1, !,
%		  open_out ".out.seq" Out,
%   		  print_ListSequentDlmt Out L1,
%   		  close_out Out,
%   		  print "Output sequent:\n",
%   		  print_ListSequent std_out L1,
%   		  open_out ".synsvr.txt" SvrOut,
%   		  output SvrOut "ok", flush SvrOut,
%   		  close_out SvrOut
%      )
%   ).

type get_rule string -> o.

get_rule Rule :-
   open_in ".rule" In,
   input_line In Str,
   N is (size Str), N' is N - 1,
   Rule is (substring Str 0 N'),
   close_in In.

serve :-    
   get_rule Rule,
   load_def ".cur.def" Def,
   (current_def Def => 
      (
          load_seq ".in.seq" [Seq|L] _, !,
          output std_out "Input sequent:\n",
          print_ListSequent std_out [Seq],     
          output std_out "Applying rule: ", print Rule, print "\n",               
          S is (substring Rule 0 3),
          print "substring : ", print S, print "\n",
          if (S = "hyp")
   		  	 (
   		  	    print "List hypothesis: ", print Rule, print "\n",
   		  	    getHyp Rule Seq LTm,
   		  	    if (LTm = nil) (fail) (true),
   		  	    open_out ".out.seq" Out,
   		  	    print_ListTermsDlmt Out LTm,
   		  	    print_ListTermsDlmt std_out LTm,
   		  	    close_out Out   		  	     
   		  	 )
             (
          		infer Rule Seq LSeq,
          		output std_out "ok\n",
          		append L LSeq L1, !,          		
		  		open_out ".out.seq" Out,
   		  		print_ListSequentDlmt Out L1,
   		  		close_out Out,
   		  		print "Output sequent:\n",
   		  		print_ListSequent std_out L1
   		  	 ),
		   open_out ".synsvr.txt" SvrOut,
   		   output SvrOut "ok", flush SvrOut,
   		   close_out SvrOut   		  	 
      )
   ).
   
type getHyp string -> sequent -> list tm -> o.

getHyp "hyp-contr-l" (seq _ _ Hyp G) Hyp.
getHyp "hyp-weak-l"  (seq _ _ Hyp G) Hyp.
getHyp "hyp-and-l"   (seq Nm LFv Hyp G) L :-
   getFoList land Hyp L.
getHyp "hyp-or-l"   (seq Nm LFv Hyp G) L :-
   getFoList lor Hyp L.
getHyp "hyp-imp-l"   (seq Nm LFv Hyp G) L :-
   getFoList limp Hyp L.
getHyp "hyp-pi-l"   (seq Nm LFv Hyp G) L :-
   getFoList lall Hyp L.
getHyp "hyp-sigma-l"   (seq Nm LFv Hyp G) L :-
   getFoList lexists Hyp L.
getHyp "hyp-eq-l"   (seq Nm LFv Hyp G) L :-
   getFoList leq Hyp L.
getHyp "hyp-def-l"   (seq Nm LFv Hyp G) L :-
   getAtomic Hyp L.
getHyp "hyp-ind-l"   (seq Nm LFv Hyp G) L :-
   getAtomic Hyp L.
   
type getFo tm -> tm -> o.
type getFoList tm -> list tm -> list tm -> o.

getFo land (land @ R @ T).
getFo lor (lor @ R @ T).
getFo limp (limp @ R @ T).
getFo leq (leq @ R @ T).
getFo lall (lall @ R).
getFo lexists (lexists @ R).

getFoList H nil nil.
getFoList H [T|L] [T|L'] :-
   getFo H T, !, getFoList H L L'.
getFoList H [T|L] L' :-
   getFoList H L L'.

type getAtomic list tm -> list tm -> o.

getAtomic nil nil.
getAtomic [T|L] [T|L'] :-
   atomic T _, !, 
   print "\n atomic formula: ", print_lterm std_out 0 T, print "\n",
   getAtomic L L'.
getAtomic [T|L] L' :-
   getAtomic L L'.
               
type infer string -> sequent -> list sequent ->  o.   

infer "init" (seq Nm LFv Hyp G)  nil :-
   member G Hyp.
   
infer "top" (seq Nm LFv Hyp ltrue) nil.

infer "bot" (seq Nm LFv Hyp G) nil :-
   member lfalse Hyp.

infer "eq-r" (seq Nm LFv Hyp (leq @ T @ T)) nil.
         
infer "and-r" (seq Nm LFv Hyp G) LSeq :-
   inf inf_andr G (seq Nm LFv Hyp G) LSeq N.
   
infer "or-r1" (seq Nm LFv Hyp G) LSeq :-
   inf inf_orr1 G (seq Nm LFv Hyp G) LSeq N.

infer "or-r2" (seq Nm LFv Hyp G) LSeq :-
   inf inf_orr2 G (seq Nm LFv Hyp G) LSeq N.
         
infer "imp-r" (seq Nm LFv Hyp G) LSeq :-
   inf inf_impr G (seq Nm LFv Hyp G) LSeq N.

infer "sigma-r" (seq Nm LFv Hyp G) LSeq :-
   inff ".in.fo" inf_sigmar G (seq Nm LFv Hyp G) LSeq N.
   
infer "pi-r" (seq Nm LFv Hyp G) LSeq :-
   inf inf_pir G (seq Nm LFv Hyp G) LSeq N.
   
infer "def-r" (seq Nm LFv Hyp G) LSeq :-
   inf inf_defr G (seq Nm LFv Hyp G) LSeq N.

infer "def-r-inst" (seq Nm LFv Hyp G) LSeq :-
   inf inf_defr_inst G (seq Nm LFv Hyp G) LSeq N.
   
infer "coind-r" (seq Nm LFv Hyp G) LSeq :-
   inff ".in.fo" inf_coindr G (seq Nm LFv Hyp G) LSeq N.
      
infer "contr-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_contrl H (seq Nm LFv Hyp' G) LSeq N.

infer "weak-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_weakl H (seq Nm LFv Hyp' G) LSeq N.

infer "and-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_andl H (seq Nm LFv Hyp' G) LSeq N.

infer "or-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_orl H (seq Nm LFv Hyp' G) LSeq N.

infer "imp-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_impl H (seq Nm LFv Hyp' G) LSeq N.

infer "sigma-l" (seq Nm LFv Hyp G) LSeq  :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_sigmal H (seq Nm LFv Hyp' G) LSeq N.

infer "pi-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inff ".in.fo" inf_pil H (seq Nm LFv Hyp' G) LSeq N.
            
infer "eq-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_eql H (seq Nm LFv Hyp' G) LSeq N.

infer "def-l" (seq Nm LFv Hyp G) LSeq :-
   choose_hyp LFv Hyp H Hyp',
   inf inf_defl H (seq Nm LFv Hyp' G) LSeq N.

infer "ind-l" (seq Nm LFv Hyp G) LSeq  :- 
   choose_hyp LFv Hyp H Hyp',
   inff ".in.fo" inf_indl H (seq Nm LFv Hyp' G) LSeq N.

infer "focus" Seq LSeq  :-
   focus Seq LSeq N.

infer "focus-eql" Seq LSeq :-
   focus_eql Seq LSeq N.
              
% --

infer "def-r-clause" (seq Nm LFv Hyp G) nil :-
   get_defr_clauses (seq Nm LFv Hyp G) LG,
   open_out ".in.fo" Out,
   print_ListTermsDlmt Out LG.

infer "def-r-one" (seq Nm LFv Hyp G) LSeq :-
   input_fterm ".main.fo" H,
   print "Clause chosen: \n",
   print_lterm std_out 0 H, print "\n",
   inf inf_defr_one H (seq Nm LFv Hyp G) LSeq N.

infer "def-r-select" (seq Nm LFv Hyp G) LSeq :-
   input_fterm ".main.fo" H,
   LSeq = [seq Nm LFv Hyp H].      
   
type choose_hyp list signat -> list tm -> tm -> list tm -> o.

choose_hyp Sig Hyp H Hyp' :-
   input_fterm ".main.fo" H,
   eval Sig H H',
   print "Hypothesis : ", print_lterm std_out 0 H',
   remove H' Hyp Hyp', print "\n ok",!.
      