% Module lincsig (signature file)
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: lincsig
% Author: Alwen Tiu
% Date: 8/26/03
% Description:
% 	This module contains common data structures used by other modules.
%   There is no definition of any of the predicates declared here in
%	the corresponding module file (lincsig.mod).
% 	Dependency : none.
% 	Used by : linc, stt, unif, lincgrammar.
%   
%	The predicate 'constant' provides a way of declaring certain symbols
%	as constant. For example, when invoking unification, we can declare certain
%   symbols as constants by using hypothetical judgments, e.g.,
%     constant X => unify ....
%

sig lincsig.

% Simply typed lambda terms, with logical constants

kind ty type.
type fo ty.
type bt string -> ty. % base types
type --> ty -> ty -> ty.
infixr --> 9.

% basic lambda terms 
kind tm type.
type lam ty -> (tm -> tm) -> tm.
type @   tm -> tm -> tm.
infixl @ 9.
type fvar string -> tm.
%type fv int -> tm.

% logical constants
type ltrue, lfalse, land, lor,
     limp, lall, lexists, leq tm.
type constant signat -> o.


% sequents 
% evars sig: hypothesis |- goal
kind signat type.
kind sequent type.
type seq string -> list signat -> list tm -> tm -> sequent.
type sg tm -> ty -> signat.

% definition clauses

kind defn type.
kind defclause type.
kind clausetype type.

% def takes the following arguments:
% - name of the definition
% - list of kind declaration
% - list of type declaration
% - list of definition clauses

type indcl, coindcl clausetype.
type def string -> list ty -> list signat -> list defclause -> defn.
type defcl clausetype -> tm -> tm -> defclause.
type current_def defn -> o.
