% Module stt 
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: stt
% Author: Alwen Tiu
% Date: 8/26/03
% Description:
%   provides basic predicates for manipulating simply typed lambda terms, 
%   e.g., evaluation, type checking, etc.

module stt.
accumulate list, control.

constant X :-
   member X 
   [sg ltrue fo, 
    sg lfalse fo, 
    sg land (fo --> fo --> fo), 
    sg lor (fo --> fo --> fo), 
    sg limp (fo --> fo --> fo), 
    sg lall ((TyAll --> fo) --> fo), 
    sg lexists ((TyEx --> fo) --> fo), 
    sg leq (TyEq --> TyEq --> fo)].
        
typeof Kn Sig X Ty :-
     member (sg X Ty) Sig, !.
typeof Kn Sig X Ty :-
     constant (sg X Ty), !.
typeof Kn Sig (S @ T) Ty :-
     typeof Kn Sig S (Ty1 --> Ty), typeof Kn Sig T Ty1.
typeof Kn Sig  (lam Ty1 L) (Ty1 --> Ty2) :-
    typeOK Kn Ty1,
	(pi x\ typeof Kn Sig x Ty1 => typeof Kn Sig (L x) Ty2).


eval Sig ( lam Ty (x\ (M @ x)) ) M' :- 
  !, eval Sig M M'.

  
eval Sig (lam Ty M) M' :-
   !, pi x\ eval Sig x x => 
               (
                  eval Sig (M x) (N x), 
                  if (N x = (N' @ x)) (M' = N') (M' = (lam Ty N))
               ).
eval Sig (M @ N) P :-
   eval Sig M M', 
   (if (M' = (lam Ty L)) 
       (eval Sig (L N) P)
       (eval Sig N N', P = (M' @ N'))
   ).
eval Sig X X :-
   (constant (sg X T1); member (sg X T2) Sig), !.
   

type termsOK list ty -> list signat -> list tm -> o.

termsOK K Sig nil.
termsOK K Sig [T|L] :-
   if (typeof K Sig T fo) 
      (true)
      (print "Hypotheses or goal must be of type o!\n", fail),
   termsOK K Sig L.
         
type typeOK list ty -> ty -> o.
type ltypeOK list ty -> list signat -> o.

typeOK LTy fo.
typeOK LTy (bt S) :- 
   if (member (bt S) LTy) (true)
      (print "Type ", print S, 
       print " undeclared!\n", fail).
typeOK LTy (A --> B) :-
   typeOK LTy A, typeOK LTy B.


ltypeOK LTy nil.
ltypeOK LTy [(sg Tm Ty)|L] :-
   typeOK LTy Ty,
   ltypeOK LTy L.

seqOK LTy Sig (seq Nm LSig Hyp G) :-
   ltypeOK LTy LSig,
   append Sig LSig Sig',
   termsOK LTy Sig' [G|Hyp].

   
type lclauseOK list ty -> list signat -> list defclause -> o.

type isfotype ty ->  o.
isfotype fo.
isfotype (A --> B) :-
   isfotype B.

lclauseOK K Sig nil.
lclauseOK K Sig [(defcl _ Head Body) | L] :-
   if (typeof K Sig Head T & isfotype T & typeof K Sig Body T)
      (lclauseOK K Sig L)
      (term_to_string Head Str, print "Definition clause for ", print Str, 
       print " is not well-typed!\n", fail).
   
defOK (def Name Kinds Sig Clauses) :-
   ltypeOK Kinds Sig, % print "type ok\n",
   lclauseOK Kinds Sig Clauses.

% ---
% Generate a fresh free variable
      
get_next_fvar nil N T :-
   term_to_string N Str,
   S is "E_" ^ Str,
   if(constant (sg (fvar S) _))
     (N' is N + 1,
      get_next_fvar nil N' T)
     (T = (fvar S)).
get_next_fvar [R|L] N T :- 
   get_next_fvar L N T',
   if (member (sg T' Ty) [R|L] ; constant (sg T' _))
      ( 
         N1 is N + 1,
         get_next_fvar [R|L] N1 T
      )
      (T = T').

% ---

linc_copy  (A @ B) (C @ D) :- linc_copy A C, linc_copy B D.
linc_copy (lam S L) (lam S M) :-
   pi x\ (linc_copy x x => (linc_copy (L x) (M x))).
linc_copy (fvar S) (fvar S).   
linc_copy ltrue ltrue.
linc_copy lfalse lfalse.
linc_copy land land.
linc_copy lor lor.
linc_copy limp limp.
linc_copy lall lall.
linc_copy lexists lexists.
linc_copy leq leq.
            